/**
 * Copyright (c) 2009-2012 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.zauberlabs.commons.mom;

import net.sf.staccatocommons.restrictions.check.NonNull;

import com.zauberlabs.commons.mom.converter.TypeConverter;
import com.zauberlabs.commons.mom.style.NamingStyle;
import com.zauberlabs.commons.mom.style.SetterStyle;

/**
 * MapObjectMapperBuilder
 *
 * @author flbulgarelli
 */
public interface MapObjectMapperBuilder {

    /**
     * Adds a new converter to the {@link MapObjectMapper} being constructed.
     *
     * @param converter
     * @return this
     */
    MapObjectMapperBuilder withConverter(@NonNull TypeConverter converter);

    /**
     * Configures the setter style of the {@link MapObjectMapper} being constructed
     * @param setterStyle
     * @return this
     */
    MapObjectMapperBuilder withSetterStyle(@NonNull SetterStyle setterStyle);

    /**
     * Configures the interceptor of the {@link MapObjectMapper} being constructed.
     *
     * @param interceptor
     * @return this
     */
    MapObjectMapperBuilder withInterceptor(@NonNull MapObjectMapperInterceptor interceptor);

    /**
     * Configures the naming style of the {@link MapObjectMapper} being constructed
     *
     * @param namingStyle
     * @return this
     */
    MapObjectMapperBuilder withNamingStyle(@NonNull NamingStyle namingStyle);

    MapObjectMapperBuilder withPropertyModel(@NonNull PropertyModel model);

    /**
     * Answers a built, new {@link MapObjectMapper}. This message may be sent multiples
     * times.
     *
     * @return a new {@link MapObjectMapper}
     */
    MapObjectMapper build();



}
