/**
 * Copyright (c) 2009-2012 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.zauberlabs.commons.mom;

import net.sf.staccatocommons.defs.Applicable;
import net.sf.staccatocommons.defs.Applicable2;
import net.sf.staccatocommons.restrictions.check.NonNull;

/**
 * {@link MapObjectMapperInterceptor} are objects capable of intercepting
 * messages sent to an {@link MapObjectMapper}, being able to surround the actual
 * method evaluation with custom code, or even replace it
 *
 * @author flbulgarelli
 */
public interface MapObjectMapperInterceptor {

    /**
     * @see com.zauberlabs.commons.mom.MapObjectMapper#map(Object)
     *
     * Then, executes the code that came in the 'proceed' parameter.
     *
     * @param value
     * @param proceed Code that will be executed.
     * @return
     */
    Object map(Object value, Applicable<Object, Object> proceed);

    /**
     * @see com.zauberlabs.commons.mom.MapObjectMapper#unmap(Object)
     *
     * Then, executes the code that came in the 'proceed' parameter.
     *
     * @param map
     * @param proceed Code that will be executed.
     * @return
     */
    Object unamp(Object map, Applicable<Object, Object> proceed);

    /**
     * @see com.zauberlabs.commons.mom.MapObjectMapper#unmap(Object, Class)
     *
     * Then, executes the code that came in the 'proceed' parameter.
     *
     * @param value
     * @param type
     * @param proceed Code that will be executed.
     * @return
     */
    Object unmap(Object value, @NonNull Class<?> type, Applicable2<Object, Class<?>, Object> proceed);

}
