/**
 * Copyright (c) 2009-2012 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.zauberlabs.commons.mom.converter.impl;


import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.xml.datatype.XMLGregorianCalendar;

import com.zauberlabs.commons.mom.converter.TypeConverter;

/**
 * {@link TypeConverter} that can convert {@link Date}s into
 * {@link XMLGregorianCalendar}s based on its time
 *
 * @author flbulgarelli
 */
public final class StringToXMLGregorianCalendarTypeConverter extends AbstractToXMLGregorianCalendarTypeConverter {

    private String format;

    /**
     * Creates a new {@link StringToXMLGregorianCalendarTypeConverter}
     *
     * @param format the date format this converter accepts
     */
    public StringToXMLGregorianCalendarTypeConverter(final String format) {
        this.format = format;
    }

    @Override
    @SuppressWarnings("unchecked")
    public <A> A convert(final Object value, final Class<A> destinationType) {
        return (A) toGregorianCalendar(parse(value));
    }

    /**parses the given date*/
    private Date parse(final Object value) {
        try {
            return new SimpleDateFormat(format).parse((String) value);
        } catch (ParseException e) {
            throw new IllegalArgumentException("Could not parse value " + value, e);
        }
    }

    @Override
    protected boolean canConvertSourceType(final Class<?> sourceType) {
        return String.class == sourceType;
    }

    public static TypeConverter from(final String format) {
        return new StringToXMLGregorianCalendarTypeConverter(format);
    }
}
