/**
 * Copyright (c) 2009-2012 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.zauberlabs.commons.mom.internal;

import net.sf.staccatocommons.defs.Applicable;
import net.sf.staccatocommons.defs.Applicable2;
import net.sf.staccatocommons.restrictions.Constant;
import net.sf.staccatocommons.restrictions.check.NonNull;

import com.zauberlabs.commons.mom.MapObjectMapperInterceptor;
import com.zauberlabs.commons.mom.PropertyModel;
import com.zauberlabs.commons.mom.converter.TypeConverter;
/**
 *
 * InterceptingMapObjectMapper
 * @author flbulgarelli
 */
public final class InterceptingMapObjectMapper extends DefaultMapObjectMapper {

    private final MapObjectMapperInterceptor interceptor;

    /***/
    public InterceptingMapObjectMapper(final String packageName, final PropertyModel propertyModel,
            final TypeConverter typeConverter, @NonNull final MapObjectMapperInterceptor interceptor) {
        super(packageName, propertyModel, typeConverter);
        this.interceptor = interceptor;
    }

    /**
     * @see com.zauberlabs.commons.mom.AbstractMapObjectMapper#map(java.lang.Object)
     */
    @Override
    public Object map(final Object value) {
        return interceptor.map(value, proceedMap());
    }

    /***/
    @Constant
    private Applicable<Object, Object> proceedMap() {
        return new Applicable<Object, Object>() {
            public Object apply(final Object arg0) {
                return InterceptingMapObjectMapper.super.map(arg0);
            }
        };
    }

    /**
     * @see com.zauberlabs.commons.mom.AbstractMapObjectMapper#unmap(java.lang.Object, java.lang.Class)
     */
    @Override
    public Object unmap(final Object value, final Class<?> type) {
        return interceptor.unmap(value, type, proceedUnamp2());
    }

    /***/
    @Constant
    private Applicable2<Object, Class<?>, Object> proceedUnamp2() {
        return new Applicable2<Object, Class<?>, Object>() {
            public Object apply(final Object arg0, final Class<?> arg1) {
                return InterceptingMapObjectMapper.super.unmap(arg0, arg1);
            }
        };
    }

    /**
     * @see com.zauberlabs.commons.mom.AbstractMapObjectMapper#unmap(java.lang.Object)
     */
    @Override
    public Object unmap(final Object map) {
        return interceptor.unamp(map, proceedUnmap());
    }

    /***/
    @Constant
    private Applicable<Object, Object> proceedUnmap() {
        return new Applicable<Object, Object>() {
            public Object apply(final Object arg0) {
                return InterceptingMapObjectMapper.super.unmap(arg0);
            }
        };
    }

}
