/**
 * Copyright (c) 2009-2012 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/*
 * Copyright (c) 2011 Zauber S.A. -- All rights reserved
 */

package com.zauberlabs.commons.mom.style.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.List;

import net.sf.staccatocommons.restrictions.check.NonNull;
import net.sf.staccatocommons.restrictions.check.NotEmpty;

import org.apache.commons.beanutils.PropertyUtils;

import com.zauberlabs.commons.mom.NaiveProperties;
import com.zauberlabs.commons.mom.style.SetterStyle;


/**
 * The Jaxb Style of setting fields.
 *
 * @author Gaston Ponti
 * @since Aug 26, 2011
 */
@SuppressWarnings("unchecked")
public class JaxbStyle extends BeanStyle {

    /**
     * An instance of {@link JaxbStyle}
     */
    public static final SetterStyle STYLE = new JaxbStyle();

    /**Answers if the type of a property is list, given a target object and propertyName **/
    private boolean isPropertyTypeList(final Object destination, final String propertyName)
            throws IllegalAccessException, InvocationTargetException,
            NoSuchMethodException {
        return List.class.isAssignableFrom(PropertyUtils.getPropertyType(
                destination, propertyName));
    }

    @Override
    public void setValue(@NonNull final Object destination, @NotEmpty final String name,
            final Object property) throws Exception {
        if (isPropertyTypeList(destination, name)) {
            if (property != null) {
                getListProperty(destination, name).addAll((List) property);
            }
        } else {
            super.setValue(destination, name, property);
        }
    }

    public List getListProperty(final Object destination, final String name) {
        return NaiveProperties.get(destination, name);
    }

}